/* This file is part of Netwib.
*/

/*-------------------------------------------------------------*/
netwib_err netwib_priv_errmsg_append_buf(netwib_constbuf *pbuf)
{
  netwib_er(netwib_priv_glovars_wrlock());
  netwib_er(netwib_buf_append_buf(pbuf, &netwib_priv_glovars.errmsg));
  netwib_er(netwib_priv_glovars_wrunlock());

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_errmsg_reinit(void)
{
  netwib_er(netwib_priv_glovars_wrlock());
  netwib__buf_reinit(&netwib_priv_glovars.errmsg);
  netwib_er(netwib_priv_glovars_wrunlock());

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_errmsg_text(netwib_conststring msg)
{
  netwib_byte array[256];
  netwib_buf buf;

  netwib_er(netwib_buf_init_ext_arrayempty(array, sizeof(array), &buf));
  netwib_er(netwib_buf_append_text(msg, &buf));

  netwib_er(netwib_priv_errmsg_reinit());
  netwib_er(netwib_priv_errmsg_append_buf(&buf));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_errmsg_func(netwib_conststring functionname,
                                   netwib_constbuf *pexplain)
{
  netwib_byte array[256];
  netwib_buf buf;

  netwib_er(netwib_buf_init_ext_arrayempty(array, sizeof(array), &buf));
  netwib_er(netwib_buf_append_text(functionname, &buf));
  netwib_er(netwib_buf_append_text(" : ", &buf));
  netwib_er(netwib_buf_append_buf(pexplain, &buf));

  netwib_er(netwib_priv_errmsg_reinit());
  netwib_er(netwib_priv_errmsg_append_buf(&buf));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_errmsg_func_text(netwib_conststring functionname,
                                        netwib_conststring explain)
{
  netwib_buf buf;

  netwib_er(netwib_buf_init_ext_text(explain, &buf));
  netwib_er(netwib_priv_errmsg_func(functionname, &buf));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_errmsg_func_unkval(netwib_conststring functionname,
                                          netwib_conststring varname)
{
  netwib_buf buf;

  netwib_er(netwib_buf_init_ext_text(varname, &buf));
  netwib_er(netwib_buf_append_text("unknown value", &buf));
  if (varname != NULL) {
    netwib_er(netwib_buf_append_text(" for ", &buf));
    netwib_er(netwib_buf_append_text(varname, &buf));
  }

  netwib_er(netwib_priv_errmsg_func(functionname, &buf));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_priv_errmsg_func_reachpt(netwib_conststring functionname)
{
  netwib_buf buf;

  netwib_er(netwib_buf_init_ext_text("we should not have reached this point",
                                   &buf));
  netwib_er(netwib_priv_errmsg_func(functionname, &buf));

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
#if defined NETWIBDEF_SYSNAME_Unix
#elif defined NETWIBDEF_SYSNAME_Windows
netwib_err netwib_priv_errmsg_path_win240(void)
{
  netwib_buf errbuf;

  netwib_er(netwib_buf_init_ext_text(" (origin of this problem is perhaps Windows'limitation of 240 chars for the total pathname length)", &errbuf));
  netwib_er(netwib_priv_errmsg_append_buf(&errbuf));

  return(NETWIB_ERR_OK);
}
#else
 #error "Unknown value for NETWIBDEF_SYSNAME"
#endif
