/*
                                  NETWIB
                             Network library
                Copyright(c) 1999-2004 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Netwib.

  Netwib is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License version 2.1 as published by the Free Software Foundation.

  Netwib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  Lesser General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <netwib/inc/maininc.h>

/*-------------------------------------------------------------*/
netwib_err netwib_uint32_init_rand(netwib_uint32 min,
                                   netwib_uint32 max,
                                   netwib_uint32 *prand)
{
  if (min > max) {
    return(NETWIB_ERR_PAINVALIDRANGE);
  }

  if (prand != NULL) {
    netwib_er(netwib_priv_uint32_init_rand(prand));
    if (min != 0 || max != 0xFFFFFFFFU) {
      *prand = min + *prand % (max - min + 1);
    }
  }

  return(NETWIB_ERR_OK);
}

/*-------------------------------------------------------------*/
netwib_err netwib_buf_append_rand(netwib_uint32 size,
                                  netwib_byte min,
                                  netwib_byte max,
                                  netwib_buf *pbuf)
{
  netwib_data data, startdata;
  netwib_uint32 r, s, range;
  netwib_byte b;

  if (min > max) {
    return(NETWIB_ERR_PAINVALIDRANGE);
  }

  netwib_er(netwib_buf_wantspace(pbuf, size, &data));
  startdata = data;

  /* chunks of 4 */
  s = size; /* we need to keep size for endoffset */
  while (s > 3) {
    netwib_er(netwib_priv_uint32_init_rand(&r));
    *data++ = netwib_c2_uint32_0(r);
    *data++ = netwib_c2_uint32_1(r);
    *data++ = netwib_c2_uint32_2(r);
    *data++ = netwib_c2_uint32_3(r);
    s -= 4;
  }

  /* last bytes */
  if (s) {
    netwib_er(netwib_priv_uint32_init_rand(&r));
    switch(s) {
    case 1 :
      *data = netwib_c2_uint32_0(r);
      break;
    case 2 :
      *data++ = netwib_c2_uint32_0(r);
      *data = netwib_c2_uint32_1(r);
      break;
    case 3 :
      *data++ = netwib_c2_uint32_0(r);
      *data++ = netwib_c2_uint32_1(r);
      *data = netwib_c2_uint32_2(r);
      break;
    }
  }

  /* eventually recompute bytes if a smaller range is expected */
  if (min != 0 || max != 255) {
    range = max - min + 1;
    s = size;
    data = startdata;
    while (s--) {
      b = *data;
      *data++ = (netwib_byte)(min + ((b*range) / 256));
    }
  }

  pbuf->endoffset += size;
  return(NETWIB_ERR_OK);
}
